/*
 *  $Id: chrecv.c,v 1.9 1994/02/02 21:15:50 gropp Exp $
 *
 *  (C) 1993 by Argonne National Laboratory and Mississipi State University.
 *      All rights reserved.  See COPYRIGHT in top-level directory.
 */


#ifndef lint
static char vcid[] = "$Id: chrecv.c,v 1.9 1994/02/02 21:15:50 gropp Exp $";
#endif /* lint */

#include "mpid.h"

static int DebugFlag = 0;

void MPID_SetDebugFlag( f )
int f;
{
DebugFlag = f;
}
void MPID_SetRecvDebugFlag( f )
int f;
{
DebugFlag = f;
}

/* 
  This file contains the routines to handle receiving a message

  Because we don't know the length of messages (at least the long ones with
  tag MPID_PT2PT2_TAG(src) tags), we never post a receive.  Rather, we have
  a MPID_ch_check_incoming routine that looks for headers.  Note that
  messages sent from a source to destination with the MPID_PT2PT2_TAG(src)
  are ordered (we assume that the message-passing system preserves order).
 */


/*
   Post a receive.

   Since the Chameleon implementation lets the underlying message transport
   layer handle this, there isn't much to do.  We DO handle the case of
   posting a BLOCKING receive by doing BLOCKING waits on incoming messages.

   Otherwise, we simply try to handle any receives that are ready for
   processing.
 */
int MPID_CH_post_recv( dmpi_recv_handle, is_available ) 
MPIR_RHANDLE *dmpi_recv_handle;
int          *is_available;
{
MPID_RHANDLE *mpid_recv_handle;
MPIR_RHANDLE *dmpi_unexpected;
int          found;

mpid_recv_handle = &dmpi_recv_handle->dev_rhandle;
*is_available    = 0;

if (DebugFlag) {
    printf( "[%d] starting recv for tag = %d, source = %d, ctx = %d\n", 
	    PImytid, dmpi_recv_handle->tag, dmpi_recv_handle->source,
	    dmpi_recv_handle->contextid );
    }
/* At this time, we check to see if the message has already been received */
MPIR_search_unexpected_queue( dmpi_recv_handle->source, 
		   dmpi_recv_handle->tag, dmpi_recv_handle->contextid, 
		   &found, 1, &dmpi_unexpected );
if (found) {
    if (DebugFlag) {
	printf( "[%d] Found message in unexpected queue\n", 
	        PImytid );
	}
    /* Copy relevant data to recv_handle */
    mpid_recv_handle           = &dmpi_unexpected->dev_rhandle;
    dmpi_recv_handle->source   = dmpi_unexpected->source;
    dmpi_recv_handle->tag      = dmpi_unexpected->tag;
    dmpi_recv_handle->totallen = mpid_recv_handle->bytes_as_contig;
    /* *dmpi_recv_handle = *dmpi_unexpected; */
    dmpi_recv_handle->dev_rhandle.is_posted = 0;
    /* Need to free unexpected */
    /* We don't do any of the transfer; we'll wait until the complete_recv
       is called */
    /* QUESTION: do we need to add to posted receive queue? */
    if (DebugFlag) {
	printf( "[%d] Found message in temp area...\n", PImytid );
	}
    DMPI_put_totallen( dmpi_recv_handle, mpid_recv_handle->bytes_as_contig );
    DMPI_put_from_contig( dmpi_recv_handle, (void *)(mpid_recv_handle->temp),
   	                  mpid_recv_handle->bytes_as_contig );
    FREE( mpid_recv_handle->temp );

    /* Now, switch to the dev handle in the previously allocated handle */
    mpid_recv_handle = &dmpi_recv_handle->dev_rhandle;

    mpid_recv_handle->temp = 0;      /* In case of a cancel */
    mpid_recv_handle->done = YES;
    DMPI_mark_recv_completed(dmpi_recv_handle);

    /* MPIR_enqueue( &posted_recvs, dmpi_recv_handle, MPIR_QRHANDLE ); */
    *is_available = 1;
    return MPI_OK;
    }

/* Add to the posted receive queue */
MPIR_enqueue( &posted_recvs, dmpi_recv_handle, MPIR_QRHANDLE );
dmpi_recv_handle->dev_rhandle.is_posted = 1;

/* If we got here, the message is not yet available */
if (!mpid_recv_handle->is_non_blocking) {
    if (DebugFlag) {
	printf( "[%d] Blocking recv; starting wait loop\n", PImytid );
	}
    while (!dmpi_recv_handle->completed) {
	(void)MPID_CH_check_incoming( MPID_BLOCKING );
	}
    }

else {
    if (DebugFlag) {
	printf( "[%d] About to do a non-blocking check of incoming messages\n",
	        PImytid );
	}
    while (MPID_CH_check_incoming( MPID_NOTBLOCKING ) != -1) ;
    }

return MPI_OK;
}

/* Check for incoming messages.
    Input Parameter:
.   is_blocking - true if this routine should block until a message is
    available

    Returns -1 if nonblocking and no messages pending
 */    
int MPID_CH_check_incoming( is_blocking )
MPID_BLOCKING_TYPE is_blocking;
{
MPID_PACKET  pkt;	
int          from;
char         *address;
MPIR_RHANDLE *dmpi_recv_handle;
int          actual_len;
MPID_RHANDLE *mpid_recv_handle;
int          is_posted;

/* If nonblocking and no headers available, exit */
if (is_blocking == MPID_NOTBLOCKING) {
    if (!PInprobe( MPID_PT2PT_TAG )) return -1;
    }

PIbrecv( MPID_PT2PT_TAG, &pkt, sizeof(MPID_PACKET), MSG_OTHER );
from = PIfrom();
/* We should check the size here for internal errors .... */

if (DebugFlag) {
    printf( "[%d] received message for tag = %d, source = %d, ctx = %d\n", 
	    PImytid, pkt.tag, from, pkt.context_id );
    }
/* Is the message expected or not? */
DMPI_msg_arrived( from, pkt.tag, pkt.context_id, 
                  &dmpi_recv_handle, &is_posted );
mpid_recv_handle = &dmpi_recv_handle->dev_rhandle;
if (is_posted) {
    DMPI_put_totallen( dmpi_recv_handle, pkt.len );
    DMPI_put_from_contig( dmpi_recv_handle, (void *)(pkt.buffer),
   	                  MPID_MIN(pkt.len,MPID_PACKET_SIZE) );
    if (DebugFlag) {
	printf( "[%d] Got data from packet\n", PImytid );
	}
    /* Handle long messages... */
    if (pkt.len > MPID_PACKET_SIZE) {
	if (DebugFlag) {
	    printf( "[%d] Receiving a long message from %d\n", PImytid, from );
	    }
        /* Question: who is responsible for this allocation? */
	DMPI_put_into_contig( dmpi_recv_handle, (void **)(&address), 
			      pkt.len - MPID_PACKET_SIZE, &actual_len );
        PIbrecv( MPID_PT2PT2_TAG(from), address, pkt.len-MPID_PACKET_SIZE,
   	         MSG_OTHER );
        }
    DMPI_mark_recv_completed(dmpi_recv_handle);
    mpid_recv_handle->done = YES;
    }
else {
    /* Note that DMPI_msg_arrived ALSO adds this message to the 
       unexpected queue */
    /* MPIR_enqueue( &unexpected_recvs, dmpi_recv_handle, MPIR_QRHANDLE ); */
    /* We buffer the message on the receiver.   There are, of course, other
       strategies */
    address = (char *)MALLOC( pkt.len ); 
    if (!address) return MPI_EXAUSTED;
    memcpy( address, pkt.buffer, MPID_MIN(pkt.len,MPID_PACKET_SIZE) );
    if (pkt.len > MPID_PACKET_SIZE) {
	/* Message has "extension" part */
        PIbrecv( MPID_PT2PT2_TAG(from), address + MPID_PACKET_SIZE, 
		 pkt.len-MPID_PACKET_SIZE, MSG_OTHER );
	}
    mpid_recv_handle->temp            = address;
    mpid_recv_handle->bytes_as_contig = pkt.len;
    }
return MPI_OK;    
}


/*
    This routine completes a particular receive.  It does this by processing
    incoming messages until the indicated message is received
 */
int MPID_CH_complete_recv( dmpi_recv_handle ) 
MPIR_RHANDLE *dmpi_recv_handle;
{
MPID_RHANDLE *mpid_recv_handle = &dmpi_recv_handle->dev_rhandle;

while (!mpid_recv_handle->done) {
    (void)MPID_CH_check_incoming( MPID_BLOCKING );
    }
#ifdef FOO
/* NEED TO IDENTIFY WHICH RECEIVE QUEUE TO WITHDRAW FROM */
if (mpid_recv_handle->is_posted) 
    MPIR_dequeue( &posted_recvs, dmpi_recv_handle );
else 
    MPIR_dequeue( &unexpected_recvs, dmpi_recv_handle );
#endif
return MPI_OK;
}
