/*
 *  $Id: red_scat.c,v 1.7 1994/02/02 21:03:58 lusk Exp $
 *
 *  (C) 1993 by Argonne National Laboratory and Mississipi State University.
 *      All rights reserved.  See COPYRIGHT in top-level directory.
 */

#ifndef lint
static char vcid[] = "$Id: red_scat.c,v 1.7 1994/02/02 21:03:58 lusk Exp $";
#endif /* lint */

#include <mpi.h>
#include "coll.h"

/*@

MPI_Reduce_scatter - 

@*/
int MPI_Reduce_scatter ( sendbuf, recvbuf, recvcnts, datatype, op, comm )
void             *sendbuf;
void             *recvbuf;
int              *recvcnts;
MPI_Datatype      datatype;
void            (*op)();
MPI_Comm          comm;
{
  int   rank, size, i, count=0, extent;
  int  *displs;
  void *buffer;
  int   errno = MPI_SUCCESS;

  /* Check for invalid arguments */
  if ( ( (comm            == (MPI_Comm)0) && (errno = MPI_ERR_COMM_NULL) )  ||
       ( (comm->comm_type == MPIR_INTER)  && (errno = MPI_ERR_COMM_INTER) ) )
    return (errno); 

  /* Determine the "count" of items to reduce and set the displacements*/
  MPI_Type_extent (datatype, &extent);
  MPI_Comm_size   (comm, &size);
  MPI_Comm_rank   (comm, &rank);
  displs = (int *)MALLOC(size*sizeof(int));
  for (i=0;i<size;i++) {
    displs[i] = count;
    count += recvcnts[i];
  }
  buffer = (void *)MALLOC(extent*count);

  /* Reduce to 0, then scatter */
  MPI_Reduce   ( sendbuf, buffer, count, datatype, op, 0, comm );
  MPI_Scatterv ( buffer, displs, recvcnts, datatype, recvbuf, recvcnts[rank],
		 datatype, datatype, 0, comm );
  
  FREE(buffer); FREE(displs);
  return (errno);
}
