/*
 *  $Id: group_util.c,v 1.6 1994/01/25 23:12:40 doss Exp $
 *
 *  (C) 1993 by Argonne National Laboratory and Mississipi State University.
 *      All rights reserved.  See COPYRIGHT in top-level directory.
 */

#include <mpi.h>


MPI_Group MPIR_CreateGroup( np )
int np;
{
    MPIR_GROUP *new;
    int        i;

    new = NEW(MPIR_GROUP);

    new->np             = np;
    new->lrank_to_grank = (int *) MALLOC( np * sizeof(int) );
    new->set_mark       = (int *)0;
    new->local_rank     = MPI_UNDEFINED;
    new->ref_count      = 1;
    MPIR_Powers_of_2 ( np, &(new->N2_next), &(new->N2_prev) );

    for (i=0; i<np; i++) 
	new->lrank_to_grank[i] = -1;

    return new;
}

void MPIR_FreeGroup( group )
MPIR_GROUP *group;
{
  FREE( group->lrank_to_grank );
  if ( group->set_mark )
    FREE( group->set_mark );
  FREE( group );
}

void MPIR_SetToIdentity( g )
MPI_Group g;
{
  int np, i;

  np = g->np;
  for (i=0; i<np; i++) 
    g->lrank_to_grank[i] = i;

  MPID_myrank(&(g->local_rank));
  if (g->local_rank > np)
    g->local_rank = MPI_UNDEFINED;
}

/*+

MPIR_GROUP_dup -

+*/
int MPIR_GROUP_dup( group, new_group )
MPI_Group group, *new_group;
{
  (*new_group) = group;
  if ( group != (MPI_Group)0 )
    group->ref_count++;
  return(MPI_SUCCESS);
}


/*+

MPIR_Dump_group - dump group information

+*/
int MPIR_Dump_group ( group )
MPI_Group group;
{
  int i, rank;

  printf ( "\tgroup       = %d\n", group );
  if (group != NULL) {
    printf ( "\tnp          = %d\n", group->np );
    printf ( "\tlocal rank  = %d\n", group->local_rank );
    printf ( "\tlocal rank -> global rank mapping\n" );
    for ( i=0; i<group->np; i++ )
      printf ( "\t    %d             %d\n", i, group->lrank_to_grank[i] );
  }
}

/*+

MPIR_Dump_ranks - dump an array of ranks

+*/
int MPIR_Dump_ranks ( n, ranks )
int n, *ranks;
{
  int i;

  printf ( "\tnumber of ranks = %d\n", n );
  printf ( "\t n     rank\n" );
  for ( i=0; i<n; i++ )
    printf ( "\t %d      %d\n", i, ranks[i] );
}

/*+

MPIR_Dump_ranges - dump an array of ranges

+*/
int MPIR_Dump_ranges ( n, ranges )
int n, *ranges;
{
  int i;

  printf ( "\tnumber of ranges = %d\n", n );
  printf ( "\t first    last    stride\n" );
  for ( i=0; i<n; i++ )
  printf ( "\t %d      %d        %d       %d\n", i, ranges[i*3],
          ranges[(i*3)+1], ranges[(i*3)+2] );
}


/*+

MPIR_Powers_of_2 - given a number N, determine the previous and next
                   powers of 2

+*/
int MPIR_Powers_of_2 ( N, N2_next, N2_prev )
int  N;
int *N2_next, *N2_prev;
{
  int high     = 131072;
  int low      = 1;

  while( (high > N) && (low < N) ) {
    high >>= 1; low  <<= 1;
  }

  if(high <= N) {
    if(high == N)   /* no defect, power of 2! */
      (*N2_next) = N;
	else
      (*N2_next) = high << 1;
  }
  else {/* condition low >= N satisfied */
    if(low == N)	/* no defect, power of 2! */
      (*N2_next) = N;
	else
      (*N2_next) = low;
  }

  if( N == (*N2_next) ) /* power of 2 */
	(*N2_prev) = N;
  else
	(*N2_prev) = (*N2_next) >> 1;

  return (MPI_SUCCESS);
}

/*+

MPIR_Group_N2_prev - retrieve greatest power of two < size of group.

+*/
int MPIR_Group_N2_prev ( group, N2_prev )
MPI_Group  group;
int       *N2_prev;
{
  (*N2_prev) = group->N2_prev;
  return (MPI_SUCCESS);
}
