/*
 *  $Id: comm_init.c,v 1.9 1994/02/03 02:29:11 lusk Exp $
 *
 *  (C) 1993 by Argonne National Laboratory and Mississipi State University.
 *      All rights reserved.  See COPYRIGHT in top-level directory.
 */

#include "mpi.h"
#include "mpisys.h"

/*@
    MPI_Comm_Init - 
@*/
int MPI_Comm_Init( handle, datatype, source_dest, tag, comm, op_mode,
		  persistence ) 
MPI_Comm_request     	 *handle;
MPI_Datatype 	 datatype;
int          	 source_dest;
int              tag;
MPI_Comm comm;
MPI_Mode         op_mode;
MPI_Persistence  persistence;
{

    MPIR_HANDLE *handleptr;
    int         found;

    if (( op_mode == MPI_STANDARD ) ||
	( op_mode == MPI_READY )    ||
	( op_mode == MPI_SYNCHRONOUS ))
    {

	*handle                        = (MPI_Comm_request) SBalloc( MPIR_shandles );
	handleptr                       = *handle;
	memset( (char *)handleptr, 0, sizeof(MPIR_SHANDLE) );
	handleptr->type                 = MPIR_SEND;
	if (source_dest == MPI_PROCNULL)
	    handleptr->shandle.dest     = source_dest;
	else
	    handleptr->shandle.dest     = comm->group->lrank_to_grank[source_dest];
	handleptr->shandle.tag          = tag;
	handleptr->shandle.contextid    = comm->send_context;
	handleptr->shandle.comm         = comm;
	handleptr->shandle.mode         = op_mode;
	handleptr->shandle.persistence  = persistence;
	handleptr->shandle.datatype     = datatype;
	handleptr->shandle.completed    = NO;
	MPID_alloc_send_handle(&((handleptr)->shandle.dev_shandle));
    }
    else if ( op_mode == MPI_RECV )
    {
	*handle                    = (MPI_Comm_request) SBalloc( MPIR_rhandles );
	handleptr                       = *handle;
	memset( (char *)handleptr, 0, sizeof(MPIR_RHANDLE) );
	handleptr->type                 = MPIR_RECV;
	if (source_dest == MPI_ANY_SOURCE)
	    handleptr->rhandle.source   = source_dest;
	else
	    handleptr->rhandle.source   = comm->group->lrank_to_grank[source_dest];
	handleptr->rhandle.tag          = tag;
	handleptr->rhandle.contextid    = comm->recv_context;
	handleptr->rhandle.comm         = comm;
	handleptr->rhandle.persistence  = persistence;
	handleptr->rhandle.datatype     = datatype;
	handleptr->rhandle.completed    = NO;
	MPID_alloc_recv_handle(&((handleptr)->rhandle.dev_rhandle));
    }
    else
	MPIR_error("bad op_mode in MPI_COMM_INIT");

    return(MPI_OK);
}
