/*
 *  $Id: cart_coords.c,v 1.5 1994/01/25 21:58:02 doss Exp $
 *
 *  (C) 1993 by Argonne National Laboratory and Mississipi State University.
 *      All rights reserved.  See COPYRIGHT in top-level directory.
 */

#include <mpi.h>

/*@

MPI_Cart_coords - determines process coords in cartesian topology given
                  rank in group

@*/
int MPI_Cart_coords ( comm, rank, maxdims, coords )
MPI_Comm  comm;
int               rank;
int               maxdims;
int              *coords;
{
  int i, ndims, *dims, num_ranks;
  int errno = MPI_SUCCESS;

  /* Check for valid arguments */
  if ( ((comm               == (MPI_Comm)0) && (errno = MPI_ERR_COMM_NULL)) ||
       ((comm->topology.type != MPI_CART)   && (errno = MPI_ERR_TOPOLOGY))  ||
       ((rank                <  0)          && (errno = MPI_ERR_RANK))      ||
       ((rank >= (comm->topology.tag.cart.nnodes))&&(errno = MPI_ERR_RANK)) ||
       ((maxdims             <  1)          && (errno = MPI_ERR_DIMS))      ||
       ((coords              == (int *)0)   && (errno = MPI_ERR_BUFFER))     )
    return (errno);

  /* Calculate coords */
  ndims     = comm->topology.tag.cart.ndims;
  dims      = comm->topology.tag.cart.dims;
  num_ranks = comm->topology.tag.cart.nnodes;
  for ( i=0; i < ndims; i++ ) {
    num_ranks = num_ranks / dims[i];
    if ( i < maxdims )
      coords[i]  = rank / num_ranks;
    rank   = rank % num_ranks;
  }

  return (errno);
}
