/*
 *  $Id: get_cart.c,v 1.6 1994/02/02 21:30:48 lusk Exp $
 *
 *  (C) 1993 by Argonne National Laboratory and Mississipi State University.
 *      All rights reserved.  See COPYRIGHT in top-level directory.
 */

#include <mpi.h>

/*@

MPI_Get_cart - retrieve cart topology information associated with a 
               communicator

@*/
int MPI_Get_cart ( comm, maxdims, dims, periods, coords )
MPI_Comm comm;
int maxdims;
int *dims, *periods, *coords;
{
  int i, num;
  int *array;
  int errno = MPI_SUCCESS;

  /* Check for valid arguments */
  if (((comm                == (MPI_Comm)0) && (errno = MPI_ERR_COMM_NULL)) ||
      ((comm->topology.type != MPI_CART)    && (errno = MPI_ERR_TOPOLOGY))   )
    return (errno);

  /* Get dims */
  num = comm->topology.tag.cart.ndims;
  array = comm->topology.tag.cart.dims;
  if ( dims != (int *)0 )
    for ( i=0; (i<maxdims) && (i<num); i++ )
      (*dims++) = (*array++);

  /* Get periods */
  num = comm->topology.tag.cart.ndims;
  array = comm->topology.tag.cart.periods;
  if ( periods != (int *)0 )
    for ( i=0; (i<maxdims) && (i<num); i++ )
      (*periods++) = (*array++);

  /* Get coords */
  num = comm->topology.tag.cart.ndims;
  array = comm->topology.tag.cart.position;
  if ( coords != (int *)0 )
    for ( i=0; (i<maxdims) && (i<num); i++ )
      (*coords++) = (*array++);

  return (errno);
}











