/*
 *  $Id: graph_nbr_cnt.c,v 1.5 1994/01/25 21:58:02 doss Exp $
 *
 *  (C) 1993 by Argonne National Laboratory and Mississipi State University.
 *      All rights reserved.  See COPYRIGHT in top-level directory.
 */

#include <mpi.h>

/*@

MPI_Graph_neighbors_count - returns the number of neighbors of a node
                            associated with a graph topology

@*/
int MPI_Graph_neighbors_count ( comm, rank, nneighbors )
MPI_Comm  comm;
int       rank;
int      *nneighbors;
{
  int errno = MPI_SUCCESS;

  /* Check for valid arguments */
  if (((comm                == (MPI_Comm)0) && (errno = MPI_ERR_COMM_NULL)) ||
      ((comm->topology.type != MPI_GRAPH)   && (errno = MPI_ERR_TOPOLOGY))  ||
      ((rank                <  0)           && (errno = MPI_ERR_RANK))      ||
      ((rank >= (comm->topology.tag.graph.nnodes))&&(errno = MPI_ERR_RANK)) ||
      ((nneighbors          == (int *)0)    && (errno = MPI_ERR_NULL))       )
    return (errno);

  /* Get nneighbors */
  if ( rank == 0 ) 
    (*nneighbors) = comm->topology.tag.graph.index[rank];
  else
    (*nneighbors) = comm->topology.tag.graph.index[rank] -
                    comm->topology.tag.graph.index[rank-1];
  
  return (errno);
}
